Module: Ruflet::UI::MaterialControlMethods

Included in:
ControlMethods
Defined in:
lib/ruflet_ui/ruflet/ui/material_control_methods.rb

Instance Method Summary collapse

Instance Method Details

#alert_dialog(**props) ⇒ Object



519
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 519

def alert_dialog(**props) = build_widget(:alertdialog, **props)

#alertdialog(**props) ⇒ Object



520
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 520

def alertdialog(**props) = alert_dialog(**props)

#animated_switcher(content = nil, **props) ⇒ Object



53
54
55
56
57
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 53

def animated_switcher(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:animatedswitcher, **mapped)
end

#animatedswitcher(content = nil, **props) ⇒ Object



58
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 58

def animatedswitcher(content = nil, **props) = animated_switcher(content, **props)

#animation(duration = nil, **props) ⇒ Object



59
60
61
62
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 59

def animation(duration = nil, **props)
  duration = props.delete(:duration) if duration.nil? && props.key?(:duration)
  Ruflet::Animation.new(duration: duration, **props)
end

#animation_style(**props) ⇒ Object



63
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 63

def animation_style(**props) = Ruflet::AnimationStyle.new(**props)

#app_bar(**props) ⇒ Object



554
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 554

def app_bar(**props) = build_widget(:appbar, **props)

#appbar(**props) ⇒ Object



555
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 555

def appbar(**props) = app_bar(**props)

#arc(**props) ⇒ Object



132
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 132

def arc(**props) = build_widget(:arc, **props)

#audio(**props) ⇒ Object



64
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 64

def audio(**props) = build_widget(:audio, **props)

#auto_complete(suggestions = nil, **props) ⇒ Object



65
66
67
68
69
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 65

def auto_complete(suggestions = nil, **props)
  mapped = props.dup
  mapped[:suggestions] = suggestions unless suggestions.nil?
  build_widget(:autocomplete, **mapped)
end

#auto_complete_suggestion(key = nil, **props) ⇒ Object



71
72
73
74
75
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 71

def auto_complete_suggestion(key = nil, **props)
  mapped = props.dup
  mapped[:key] = key unless key.nil?
  build_widget(:autocompletesuggestion, **mapped)
end

#autocomplete(suggestions = nil, **props) ⇒ Object



70
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 70

def autocomplete(suggestions = nil, **props) = auto_complete(suggestions, **props)

#autocomplete_suggestion(key = nil, **props) ⇒ Object



76
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 76

def autocomplete_suggestion(key = nil, **props) = auto_complete_suggestion(key, **props)

#autocompletesuggestion(key = nil, **props) ⇒ Object



77
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 77

def autocompletesuggestion(key = nil, **props) = auto_complete_suggestion(key, **props)

#autofill_group(content = nil, **props) ⇒ Object



84
85
86
87
88
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 84

def autofill_group(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:autofillgroup, **mapped)
end

#autofillgroup(content = nil, **props) ⇒ Object



89
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 89

def autofillgroup(content = nil, **props) = autofill_group(content, **props)

#badge(label = nil, **props) ⇒ Object



388
389
390
391
392
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 388

def badge(label = nil, **props)
  mapped = props.dup
  mapped[:label] = label unless label.nil?
  build_widget(:badge, **mapped)
end


404
405
406
407
408
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 404

def banner(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:banner, **mapped)
end

#bar_chart(**props) ⇒ Object



600
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 600

def bar_chart(**props) = build_widget(:barchart, **props)

#bar_chart_group(**props) ⇒ Object



602
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 602

def bar_chart_group(**props) = build_widget(:barchartgroup, **props)

#bar_chart_rod(**props) ⇒ Object



604
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 604

def bar_chart_rod(**props) = build_widget(:barchartrod, **props)

#bar_chart_rod_stack_item(**props) ⇒ Object



606
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 606

def bar_chart_rod_stack_item(**props) = build_widget(:barchartrodstackitem, **props)

#barchart(**props) ⇒ Object



601
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 601

def barchart(**props) = bar_chart(**props)

#barchartgroup(**props) ⇒ Object



603
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 603

def barchartgroup(**props) = bar_chart_group(**props)

#barchartrod(**props) ⇒ Object



605
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 605

def barchartrod(**props) = bar_chart_rod(**props)

#barchartrodstackitem(**props) ⇒ Object



607
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 607

def barchartrodstackitem(**props) = bar_chart_rod_stack_item(**props)

#bottom_app_bar(content = nil, **props) ⇒ Object



409
410
411
412
413
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 409

def bottom_app_bar(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:bottomappbar, **mapped)
end

#bottom_sheet(content = nil, **props) ⇒ Object



527
528
529
530
531
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 527

def bottom_sheet(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:bottomsheet, **mapped)
end

#bottomappbar(content = nil, **props) ⇒ Object



414
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 414

def bottomappbar(content = nil, **props) = bottom_app_bar(content, **props)

#bottomsheet(content = nil, **props) ⇒ Object



532
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 532

def bottomsheet(content = nil, **props) = bottom_sheet(content, **props)

#button(content = nil, **props) ⇒ Object



422
423
424
425
426
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 422

def button(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:button, **mapped)
end

#candlestick_chart(**props) ⇒ Object



618
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 618

def candlestick_chart(**props) = build_widget(:candlestickchart, **props)

#candlestick_chart_spot(**props) ⇒ Object



620
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 620

def candlestick_chart_spot(**props) = build_widget(:candlestickchartspot, **props)

#candlestickchart(**props) ⇒ Object



619
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 619

def candlestickchart(**props) = candlestick_chart(**props)

#candlestickchartspot(**props) ⇒ Object



621
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 621

def candlestickchartspot(**props) = candlestick_chart_spot(**props)

#canvas(shapes = nil, **props) ⇒ Object



125
126
127
128
129
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 125

def canvas(shapes = nil, **props)
  mapped = props.dup
  mapped[:shapes] = shapes unless shapes.nil?
  build_widget(:canvas, **mapped)
end

#canvas_color(**props) ⇒ Object



134
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 134

def canvas_color(**props) = color(**props)

#card(content = nil, **props) ⇒ Object



168
169
170
171
172
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 168

def card(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:card, **mapped)
end

#center(**props, &block) ⇒ Object



17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 17

def center(**props, &block)
  mapped = props.dup
  defaults = { expand: true, alignment: { x: 0, y: 0 } }

  if block
    nested = WidgetBuilder.new
    block_result = nested.instance_eval(&block)
    child =
      if nested.children.any?
        nested.children.first
      elsif block_result.is_a?(Ruflet::Control)
        block_result
      end
    mapped[:content] = child if child
  end

  build_widget(:container, **normalize_container_props(defaults.merge(mapped)))
end

#chart_axis(**props) ⇒ Object



634
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 634

def chart_axis(**props) = build_widget(:chartaxis, **props)

#chart_axis_label(**props) ⇒ Object



636
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 636

def chart_axis_label(**props) = build_widget(:chartaxislabel, **props)

#chartaxis(**props) ⇒ Object



635
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 635

def chartaxis(**props) = chart_axis(**props)

#chartaxislabel(**props) ⇒ Object



637
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 637

def chartaxislabel(**props) = chart_axis_label(**props)

#checkbox(**props) ⇒ Object



503
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 503

def checkbox(**props) = build_widget(:checkbox, **props)

#chip(label = nil, **props) ⇒ Object



393
394
395
396
397
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 393

def chip(label = nil, **props)
  mapped = props.dup
  mapped[:label] = label unless label.nil?
  build_widget(:chip, **mapped)
end

#circle(**props) ⇒ Object



131
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 131

def circle(**props) = build_widget(:circle, **props)

#circle_avatar(content = nil, **props) ⇒ Object



398
399
400
401
402
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 398

def circle_avatar(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:circleavatar, **mapped)
end

#circle_layer(circles = nil, **props) ⇒ Object



193
194
195
196
197
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 193

def circle_layer(circles = nil, **props)
  mapped = props.dup
  mapped[:circles] = circles unless circles.nil?
  build_widget(:circlelayer, **mapped)
end

#circle_marker(**props) ⇒ Object



199
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 199

def circle_marker(**props) = build_widget(:circlemarker, **props)

#circleavatar(content = nil, **props) ⇒ Object



403
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 403

def circleavatar(content = nil, **props) = circle_avatar(content, **props)

#circlelayer(circles = nil, **props) ⇒ Object



198
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 198

def circlelayer(circles = nil, **props) = circle_layer(circles, **props)

#circlemarker(**props) ⇒ Object



200
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 200

def circlemarker(**props) = circle_marker(**props)

#clipboard(**props) ⇒ Object



557
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 557

def clipboard(**props) = build_widget(:clipboard, **props)

#code_editor(value = nil, **props) ⇒ Object



654
655
656
657
658
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 654

def code_editor(value = nil, **props)
  mapped = props.dup
  mapped[:value] = value unless value.nil?
  build_widget(:codeeditor, **mapped)
end

#codeeditor(value = nil, **props) ⇒ Object



659
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 659

def codeeditor(value = nil, **props) = code_editor(value, **props)

#color(**props) ⇒ Object



133
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 133

def color(**props) = build_widget(:color, **props)

#column(children = nil, **props, &block) ⇒ Object



11
12
13
14
15
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 11

def column(children = nil, **props, &block)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:column, **mapped, &block)
end

#container(**props, &block) ⇒ Object



52
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 52

def container(**props, &block) = build_widget(:container, **normalize_container_props(props), &block)

#context_menu(content = nil, **props) ⇒ Object



78
79
80
81
82
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 78

def context_menu(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:contextmenu, **mapped)
end

#contextmenu(content = nil, **props) ⇒ Object



83
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 83

def contextmenu(content = nil, **props) = context_menu(content, **props)

#data_cell(content = nil, **props) ⇒ Object



280
281
282
283
284
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 280

def data_cell(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:datacell, **mapped)
end

#data_column(label = nil, **props) ⇒ Object



268
269
270
271
272
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 268

def data_column(label = nil, **props)
  mapped = props.dup
  mapped[:label] = label unless label.nil?
  build_widget(:datacolumn, **mapped)
end

#data_row(cells = nil, **props) ⇒ Object



274
275
276
277
278
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 274

def data_row(cells = nil, **props)
  mapped = props.dup
  mapped[:cells] = cells unless cells.nil?
  build_widget(:datarow, **mapped)
end

#data_table(columns = nil, **props) ⇒ Object



262
263
264
265
266
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 262

def data_table(columns = nil, **props)
  mapped = props.dup
  mapped[:columns] = columns unless columns.nil?
  build_widget(:datatable, **mapped)
end

#datacell(content = nil, **props) ⇒ Object



285
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 285

def datacell(content = nil, **props) = data_cell(content, **props)

#datacolumn(label = nil, **props) ⇒ Object



273
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 273

def datacolumn(label = nil, **props) = data_column(label, **props)

#datarow(cells = nil, **props) ⇒ Object



279
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 279

def datarow(cells = nil, **props) = data_row(cells, **props)

#datatable(columns = nil, **props) ⇒ Object



267
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 267

def datatable(columns = nil, **props) = data_table(columns, **props)

#date_picker(**props) ⇒ Object



258
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 258

def date_picker(**props) = build_widget(:datepicker, **props)

#date_range_picker(**props) ⇒ Object



260
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 260

def date_range_picker(**props) = build_widget(:daterangepicker, **props)

#datepicker(**props) ⇒ Object



259
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 259

def datepicker(**props) = date_picker(**props)

#daterangepicker(**props) ⇒ Object



261
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 261

def daterangepicker(**props) = date_range_picker(**props)

#dismissible(content = nil, **props) ⇒ Object



157
158
159
160
161
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 157

def dismissible(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:dismissible, **mapped)
end

#divider(**props) ⇒ Object



249
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 249

def divider(**props) = build_widget(:divider, **props)

#drag_target(content = nil, **props, &block) ⇒ Object



162
163
164
165
166
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 162

def drag_target(content = nil, **props, &block)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:dragtarget, **mapped, &block)
end

#draggable(content = nil, **props, &block) ⇒ Object



152
153
154
155
156
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 152

def draggable(content = nil, **props, &block)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:draggable, **mapped, &block)
end

#dragtarget(content = nil, **props, &block) ⇒ Object



167
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 167

def dragtarget(content = nil, **props, &block) = drag_target(content, **props, &block)


300
301
302
303
304
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 300

def dropdown(options = nil, **props)
  mapped = props.dup
  mapped[:options] = options unless options.nil?
  build_widget(:dropdown, **mapped)
end


311
312
313
314
315
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 311

def dropdown_m2(options = nil, **props)
  mapped = props.dup
  mapped[:options] = options unless options.nil?
  build_widget(:dropdownm2, **mapped)
end


305
306
307
308
309
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 305

def dropdown_option(key = nil, **props)
  mapped = props.dup
  mapped[:key] = key unless key.nil?
  build_widget(:dropdownoption, **mapped)
end


316
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 316

def dropdownm2(options = nil, **props) = dropdown_m2(options, **props)


310
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 310

def dropdownoption(key = nil, **props) = dropdown_option(key, **props)

#elevated_button(content = nil, **props) ⇒ Object

Ruflet currently uses a single Material button control schema. Keep elevated_button DSL available by routing to :button.



429
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 429

def elevated_button(content = nil, **props) = button(content, **props)

#expansion_panel(**props) ⇒ Object



292
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 292

def expansion_panel(**props) = build_widget(:expansionpanel, **props)

#expansion_panel_list(children = nil, **props) ⇒ Object



294
295
296
297
298
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 294

def expansion_panel_list(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:expansionpanellist, **mapped)
end

#expansion_tile(children = nil, **props) ⇒ Object



286
287
288
289
290
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 286

def expansion_tile(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:expansiontile, **mapped)
end

#expansionpanel(**props) ⇒ Object



293
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 293

def expansionpanel(**props) = expansion_panel(**props)

#expansionpanellist(children = nil, **props) ⇒ Object



299
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 299

def expansionpanellist(children = nil, **props) = expansion_panel_list(children, **props)

#expansiontile(children = nil, **props) ⇒ Object



291
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 291

def expansiontile(children = nil, **props) = expansion_tile(children, **props)

#fab(content = nil, **props) ⇒ Object



667
668
669
670
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 667

def fab(content = nil, **props)
  mapped = normalize_fab_props(props.dup, content)
  build_widget(:floatingactionbutton, **mapped)
end

#fill(**props) ⇒ Object



135
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 135

def fill(**props) = build_widget(:fill, **props)

#filled_button(content = nil, **props) ⇒ Object



436
437
438
439
440
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 436

def filled_button(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:filledbutton, **mapped)
end

#filled_icon_button(icon = nil, **props) ⇒ Object



442
443
444
445
446
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 442

def filled_icon_button(icon = nil, **props)
  mapped = props.dup
  mapped[:icon] = icon unless icon.nil?
  build_widget(:fillediconbutton, **mapped)
end

#filled_tonal_button(content = nil, **props) ⇒ Object



448
449
450
451
452
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 448

def filled_tonal_button(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:filledtonalbutton, **mapped)
end

#filled_tonal_icon_button(icon = nil, **props) ⇒ Object



454
455
456
457
458
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 454

def filled_tonal_icon_button(icon = nil, **props)
  mapped = props.dup
  mapped[:icon] = icon unless icon.nil?
  build_widget(:filledtonaliconbutton, **mapped)
end

#filledbutton(content = nil, **props) ⇒ Object



441
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 441

def filledbutton(content = nil, **props) = filled_button(content, **props)

#fillediconbutton(icon = nil, **props) ⇒ Object



447
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 447

def fillediconbutton(icon = nil, **props) = filled_icon_button(icon, **props)

#filledtonalbutton(content = nil, **props) ⇒ Object



453
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 453

def filledtonalbutton(content = nil, **props) = filled_tonal_button(content, **props)

#filledtonaliconbutton(icon = nil, **props) ⇒ Object



459
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 459

def filledtonaliconbutton(icon = nil, **props) = filled_tonal_icon_button(icon, **props)

#floating_action_button(**props) ⇒ Object



558
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 558

def floating_action_button(**props) = build_widget(:floatingactionbutton, **props)

#floatingactionbutton(**props) ⇒ Object



559
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 559

def floatingactionbutton(**props) = floating_action_button(**props)

#gesture_detector(**props, &block) ⇒ Object



123
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 123

def gesture_detector(**props, &block) = build_widget(:gesturedetector, **props, &block)

#gesturedetector(**props, &block) ⇒ Object



124
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 124

def gesturedetector(**props, &block) = gesture_detector(**props, &block)

#grid_view(children = nil, **props, &block) ⇒ Object



46
47
48
49
50
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 46

def grid_view(children = nil, **props, &block)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:gridview, **mapped, &block)
end

#gridview(children = nil, **props, &block) ⇒ Object



51
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 51

def gridview(children = nil, **props, &block) = grid_view(children, **props, &block)

#hero(content = nil, **props) ⇒ Object



90
91
92
93
94
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 90

def hero(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:hero, **mapped)
end

#icon(icon = nil, **props) ⇒ Object



540
541
542
543
544
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 540

def icon(icon = nil, **props)
  mapped = props.dup
  mapped[:icon] = icon unless icon.nil?
  build_widget(:icon, **mapped)
end

#icon_button(icon = nil, **props) ⇒ Object



473
474
475
476
477
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 473

def icon_button(icon = nil, **props)
  mapped = props.dup
  mapped[:icon] = icon unless icon.nil?
  build_widget(:iconbutton, **mapped)
end

#iconbutton(icon = nil, **props) ⇒ Object



478
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 478

def iconbutton(icon = nil, **props) = icon_button(icon, **props)

#image(src = nil, src_base64: nil, placeholder_src: nil, **props) ⇒ Object



546
547
548
549
550
551
552
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 546

def image(src = nil, src_base64: nil, placeholder_src: nil, **props)
  mapped = props.dup
  mapped[:src] = normalize_image_source(src) unless src.nil?
  mapped[:src_base64] = normalize_image_source(src_base64) unless src_base64.nil?
  mapped[:placeholder_src] = normalize_image_source(placeholder_src) unless placeholder_src.nil?
  build_widget(:image, **mapped)
end

#interactive_viewer(content = nil, **props) ⇒ Object



479
480
481
482
483
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 479

def interactive_viewer(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:interactiveviewer, **mapped)
end

#interactiveviewer(content = nil, **props) ⇒ Object



484
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 484

def interactiveviewer(content = nil, **props) = interactive_viewer(content, **props)

#keyboard_listener(content = nil, **props) ⇒ Object



117
118
119
120
121
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 117

def keyboard_listener(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:keyboardlistener, **mapped)
end

#keyboardlistener(content = nil, **props) ⇒ Object



122
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 122

def keyboardlistener(content = nil, **props) = keyboard_listener(content, **props)

#line(**props) ⇒ Object



130
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 130

def line(**props) = build_widget(:line, **props)

#line_chart(**props) ⇒ Object



608
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 608

def line_chart(**props) = build_widget(:linechart, **props)

#line_chart_data(**props) ⇒ Object



610
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 610

def line_chart_data(**props) = build_widget(:linechartdata, **props)

#line_chart_data_point(**props) ⇒ Object



612
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 612

def line_chart_data_point(**props) = build_widget(:linechartdatapoint, **props)

#linechart(**props) ⇒ Object



609
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 609

def linechart(**props) = line_chart(**props)

#linechartdata(**props) ⇒ Object



611
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 611

def linechartdata(**props) = line_chart_data(**props)

#linechartdatapoint(**props) ⇒ Object



613
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 613

def linechartdatapoint(**props) = line_chart_data_point(**props)

#list_tile(**props) ⇒ Object



173
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 173

def list_tile(**props) = build_widget(:listtile, **props)

#list_view(children = nil, **props) ⇒ Object



219
220
221
222
223
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 219

def list_view(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:listview, **mapped)
end

#listtile(**props) ⇒ Object



174
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 174

def listtile(**props) = list_tile(**props)

#listview(children = nil, **props) ⇒ Object



224
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 224

def listview(children = nil, **props) = list_view(children, **props)

#map(layers = nil, **props) ⇒ Object



175
176
177
178
179
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 175

def map(layers = nil, **props)
  mapped = props.dup
  mapped[:layers] = layers unless layers.nil?
  build_widget(:map, **mapped)
end

#markdown(value = nil, **props) ⇒ Object



534
535
536
537
538
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 534

def markdown(value = nil, **props)
  mapped = props.dup
  mapped[:value] = value unless value.nil?
  build_widget(:markdown, **mapped)
end

#marker(content = nil, **props) ⇒ Object



188
189
190
191
192
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 188

def marker(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:marker, **mapped)
end

#marker_layer(markers = nil, **props) ⇒ Object



182
183
184
185
186
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 182

def marker_layer(markers = nil, **props)
  mapped = props.dup
  mapped[:markers] = markers unless markers.nil?
  build_widget(:markerlayer, **mapped)
end

#markerlayer(markers = nil, **props) ⇒ Object



187
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 187

def markerlayer(markers = nil, **props) = marker_layer(markers, **props)


225
226
227
228
229
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 225

def menu_bar(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:menubar, **mapped)
end


231
232
233
234
235
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 231

def menu_item_button(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:menuitembutton, **mapped)
end


230
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 230

def menubar(children = nil, **props) = menu_bar(children, **props)


236
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 236

def menuitembutton(content = nil, **props) = menu_item_button(content, **props)

#merge_semantics(content = nil, **props) ⇒ Object



237
238
239
240
241
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 237

def merge_semantics(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:mergesemantics, **mapped)
end

#mergesemantics(content = nil, **props) ⇒ Object



242
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 242

def mergesemantics(content = nil, **props) = merge_semantics(content, **props)


584
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 584

def navigation_bar(**props, &block) = build_widget(:navigationbar, **props, &block)


586
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 586

def navigation_bar_destination(**props, &block) = build_widget(:navigationbardestination, **props, &block)


592
593
594
595
596
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 592

def navigation_drawer(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:navigationdrawer, **mapped)
end


598
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 598

def navigation_drawer_destination(**props) = build_widget(:navigationdrawerdestination, **props)


588
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 588

def navigation_rail(**props, &block) = build_widget(:navigationrail, **props, &block)


590
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 590

def navigation_rail_destination(**props, &block) = build_widget(:navigationraildestination, **props, &block)


585
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 585

def navigationbar(**props, &block) = navigation_bar(**props, &block)


587
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 587

def navigationbardestination(**props, &block) = navigation_bar_destination(**props, &block)


597
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 597

def navigationdrawer(children = nil, **props) = navigation_drawer(children, **props)


599
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 599

def navigationdrawerdestination(**props) = navigation_drawer_destination(**props)


589
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 589

def navigationrail(**props, &block) = navigation_rail(**props, &block)


591
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 591

def navigationraildestination(**props, &block) = navigation_rail_destination(**props, &block)

#outlined_button(content = nil, **props) ⇒ Object



460
461
462
463
464
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 460

def outlined_button(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:outlinedbutton, **mapped)
end

#outlined_icon_button(icon = nil, **props) ⇒ Object



466
467
468
469
470
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 466

def outlined_icon_button(icon = nil, **props)
  mapped = props.dup
  mapped[:icon] = icon unless icon.nil?
  build_widget(:outlinediconbutton, **mapped)
end

#outlinedbutton(content = nil, **props) ⇒ Object



465
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 465

def outlinedbutton(content = nil, **props) = outlined_button(content, **props)

#outlinediconbutton(icon = nil, **props) ⇒ Object



471
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 471

def outlinediconbutton(icon = nil, **props) = outlined_icon_button(icon, **props)

#oval(**props) ⇒ Object



136
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 136

def oval(**props) = build_widget(:oval, **props)

#overlay(children = nil, **props) ⇒ Object



95
96
97
98
99
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 95

def overlay(children = nil, **props)
  mapped = props.dup
  mapped[:controls] = children unless children.nil?
  build_widget(:overlay, **mapped)
end

#page_view(children = nil, **props) ⇒ Object



324
325
326
327
328
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 324

def page_view(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:pageview, **mapped)
end

#pageview(children = nil, **props) ⇒ Object



329
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 329

def pageview(children = nil, **props) = page_view(children, **props)

#paint(**props) ⇒ Object



141
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 141

def paint(**props) = drawing_payload(props)

#path(**props) ⇒ Object



139
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 139

def path(**props) = build_widget(:path, **props)

#path_arc(**props) ⇒ Object



144
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 144

def path_arc(**props) = drawing_payload(props.merge(_type: "Arc"))

#path_arc_to(**props) ⇒ Object



145
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 145

def path_arc_to(**props) = drawing_payload(props.merge(_type: "ArcTo"))

#path_close(**props) ⇒ Object



151
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 151

def path_close(**props) = drawing_payload(props.merge(_type: "Close"))

#path_cubic_to(**props) ⇒ Object



149
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 149

def path_cubic_to(**props) = drawing_payload(props.merge(_type: "CubicTo"))

#path_line_to(x = nil, y = nil, **props) ⇒ Object



143
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 143

def path_line_to(x = nil, y = nil, **props) = path_point_payload("LineTo", x, y, props)

#path_move_to(x = nil, y = nil, **props) ⇒ Object



142
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 142

def path_move_to(x = nil, y = nil, **props) = path_point_payload("MoveTo", x, y, props)

#path_oval(**props) ⇒ Object



146
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 146

def path_oval(**props) = drawing_payload(props.merge(_type: "Oval"))

#path_quadratic_to(**props) ⇒ Object



148
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 148

def path_quadratic_to(**props) = drawing_payload(props.merge(_type: "QuadraticTo"))

#path_rect(**props) ⇒ Object



147
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 147

def path_rect(**props) = drawing_payload(props.merge(_type: "Rect"))

#path_sub_path(**props) ⇒ Object



150
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 150

def path_sub_path(**props) = drawing_payload(props.merge(_type: "SubPath"))

#pie_chart(**props) ⇒ Object



614
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 614

def pie_chart(**props) = build_widget(:piechart, **props)

#pie_chart_section(**props) ⇒ Object



616
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 616

def pie_chart_section(**props) = build_widget(:piechartsection, **props)

#piechart(**props) ⇒ Object



615
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 615

def piechart(**props) = pie_chart(**props)

#piechartsection(**props) ⇒ Object



617
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 617

def piechartsection(**props) = pie_chart_section(**props)

#placeholder(content = nil, **props) ⇒ Object



319
320
321
322
323
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 319

def placeholder(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:placeholder, **mapped)
end

#points(**props) ⇒ Object



137
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 137

def points(**props) = build_widget(:points, **props)

#polygon_layer(polygons = nil, **props) ⇒ Object



209
210
211
212
213
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 209

def polygon_layer(polygons = nil, **props)
  mapped = props.dup
  mapped[:polygons] = polygons unless polygons.nil?
  build_widget(:polygonlayer, **mapped)
end

#polygon_marker(**props) ⇒ Object



215
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 215

def polygon_marker(**props) = build_widget(:polygonmarker, **props)

#polygonlayer(polygons = nil, **props) ⇒ Object



214
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 214

def polygonlayer(polygons = nil, **props) = polygon_layer(polygons, **props)

#polygonmarker(**props) ⇒ Object



216
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 216

def polygonmarker(**props) = polygon_marker(**props)

#polyline_layer(polylines = nil, **props) ⇒ Object



201
202
203
204
205
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 201

def polyline_layer(polylines = nil, **props)
  mapped = props.dup
  mapped[:polylines] = polylines unless polylines.nil?
  build_widget(:polylinelayer, **mapped)
end

#polyline_marker(**props) ⇒ Object



207
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 207

def polyline_marker(**props) = build_widget(:polylinemarker, **props)

#polylinelayer(polylines = nil, **props) ⇒ Object



206
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 206

def polylinelayer(polylines = nil, **props) = polyline_layer(polylines, **props)

#polylinemarker(**props) ⇒ Object



208
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 208

def polylinemarker(**props) = polyline_marker(**props)


485
486
487
488
489
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 485

def popup_menu_button(items = nil, **props)
  mapped = props.dup
  mapped[:items] = items unless items.nil?
  build_widget(:popupmenubutton, **mapped)
end


491
492
493
494
495
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 491

def popup_menu_item(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:popupmenuitem, **mapped)
end

#popupmenubutton(items = nil, **props) ⇒ Object



490
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 490

def popupmenubutton(items = nil, **props) = popup_menu_button(items, **props)

#popupmenuitem(content = nil, **props) ⇒ Object



496
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 496

def popupmenuitem(content = nil, **props) = popup_menu_item(content, **props)

#progress_bar(**props) ⇒ Object



317
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 317

def progress_bar(**props) = build_widget(:progressbar, **props)

#progress_ring(**props) ⇒ Object



330
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 330

def progress_ring(**props) = build_widget(:progressring, **props)

#progressbar(**props) ⇒ Object



318
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 318

def progressbar(**props) = progress_bar(**props)

#progressring(**props) ⇒ Object



331
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 331

def progressring(**props) = progress_ring(**props)

#radar_chart(**props) ⇒ Object



622
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 622

def radar_chart(**props) = build_widget(:radarchart, **props)

#radar_chart_title(**props) ⇒ Object



624
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 624

def radar_chart_title(**props) = build_widget(:radarcharttitle, **props)

#radar_data_set(**props) ⇒ Object



626
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 626

def radar_data_set(**props) = build_widget(:radardataset, **props)

#radar_data_set_entry(**props) ⇒ Object



628
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 628

def radar_data_set_entry(**props) = build_widget(:radardatasetentry, **props)

#radarchart(**props) ⇒ Object



623
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 623

def radarchart(**props) = radar_chart(**props)

#radarcharttitle(**props) ⇒ Object



625
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 625

def radarcharttitle(**props) = radar_chart_title(**props)

#radardataset(**props) ⇒ Object



627
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 627

def radardataset(**props) = radar_data_set(**props)

#radardatasetentry(**props) ⇒ Object



629
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 629

def radardatasetentry(**props) = radar_data_set_entry(**props)

#radio(**props) ⇒ Object



512
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 512

def radio(**props) = build_widget(:radio, **props)

#radio_group(content = nil, **props) ⇒ Object



513
514
515
516
517
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 513

def radio_group(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:radiogroup, **mapped)
end

#radiogroup(content = nil, **props) ⇒ Object



518
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 518

def radiogroup(content = nil, **props) = radio_group(content, **props)

#range_slider(**props) ⇒ Object



332
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 332

def range_slider(**props) = build_widget(:rangeslider, **props)

#rangeslider(**props) ⇒ Object



333
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 333

def rangeslider(**props) = range_slider(**props)

#rect(**props) ⇒ Object



138
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 138

def rect(**props) = build_widget(:rect, **props)

#reorderable_drag_handle(content = nil, **props) ⇒ Object



340
341
342
343
344
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 340

def reorderable_drag_handle(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:reorderabledraghandle, **mapped)
end

#reorderable_list_view(children = nil, **props) ⇒ Object



346
347
348
349
350
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 346

def reorderable_list_view(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:reorderablelistview, **mapped)
end

#reorderabledraghandle(content = nil, **props) ⇒ Object



345
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 345

def reorderabledraghandle(content = nil, **props) = reorderable_drag_handle(content, **props)

#reorderablelistview(children = nil, **props) ⇒ Object



351
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 351

def reorderablelistview(children = nil, **props) = reorderable_list_view(children, **props)

#responsive_row(children = nil, **props, &block) ⇒ Object



334
335
336
337
338
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 334

def responsive_row(children = nil, **props, &block)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:responsiverow, **mapped, &block)
end

#responsiverow(children = nil, **props, &block) ⇒ Object



339
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 339

def responsiverow(children = nil, **props, &block) = responsive_row(children, **props, &block)

#rive(src = nil, **props) ⇒ Object



661
662
663
664
665
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 661

def rive(src = nil, **props)
  mapped = props.dup
  mapped[:src] = src unless src.nil?
  build_widget(:rive, **mapped)
end

#row(children = nil, **props, &block) ⇒ Object



36
37
38
39
40
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 36

def row(children = nil, **props, &block)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:row, **mapped, &block)
end

#safe_area(content = nil, **props) ⇒ Object



352
353
354
355
356
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 352

def safe_area(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:safearea, **mapped)
end

#safearea(content = nil, **props) ⇒ Object



357
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 357

def safearea(content = nil, **props) = safe_area(content, **props)

#scatter_chart(**props) ⇒ Object



630
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 630

def scatter_chart(**props) = build_widget(:scatterchart, **props)

#scatter_chart_spot(**props) ⇒ Object



632
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 632

def scatter_chart_spot(**props) = build_widget(:scatterchartspot, **props)

#scatterchart(**props) ⇒ Object



631
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 631

def scatterchart(**props) = scatter_chart(**props)

#scatterchartspot(**props) ⇒ Object



633
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 633

def scatterchartspot(**props) = scatter_chart_spot(**props)

#search_bar(children = nil, **props) ⇒ Object



375
376
377
378
379
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 375

def search_bar(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:searchbar, **mapped)
end

#searchbar(children = nil, **props) ⇒ Object



380
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 380

def searchbar(children = nil, **props) = search_bar(children, **props)

#segment(value = nil, **props) ⇒ Object



358
359
360
361
362
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 358

def segment(value = nil, **props)
  mapped = props.dup
  mapped[:value] = value unless value.nil?
  build_widget(:segment, **mapped)
end

#segmented_button(segments = nil, **props) ⇒ Object



363
364
365
366
367
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 363

def segmented_button(segments = nil, **props)
  mapped = props.dup
  mapped[:segments] = segments unless segments.nil?
  build_widget(:segmentedbutton, **mapped)
end

#segmentedbutton(segments = nil, **props) ⇒ Object



368
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 368

def segmentedbutton(segments = nil, **props) = segmented_button(segments, **props)

#selection_area(content = nil, **props) ⇒ Object



369
370
371
372
373
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 369

def selection_area(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:selectionarea, **mapped)
end

#selectionarea(content = nil, **props) ⇒ Object



374
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 374

def selectionarea(content = nil, **props) = selection_area(content, **props)

#semantics(content = nil, **props) ⇒ Object



381
382
383
384
385
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 381

def semantics(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:semantics, **mapped)
end

#shader_mask(content = nil, **props) ⇒ Object



100
101
102
103
104
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 100

def shader_mask(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:shadermask, **mapped)
end

#shadermask(content = nil, **props) ⇒ Object



105
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 105

def shadermask(content = nil, **props) = shader_mask(content, **props)

#shadow(**props) ⇒ Object



140
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 140

def shadow(**props) = build_widget(:shadow, **props)

#shimmer(content = nil, **props) ⇒ Object



106
107
108
109
110
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 106

def shimmer(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:shimmer, **mapped)
end

#simple_attribution(**props) ⇒ Object



217
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 217

def simple_attribution(**props) = build_widget(:simpleattribution, **props)

#simpleattribution(**props) ⇒ Object



218
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 218

def simpleattribution(**props) = simple_attribution(**props)

#slider(**props) ⇒ Object



505
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 505

def slider(**props) = build_widget(:slider, **props)

#snack_bar(content = nil, **props) ⇒ Object



521
522
523
524
525
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 521

def snack_bar(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:snackbar, **mapped)
end

#snackbar(content = nil, **props) ⇒ Object



526
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 526

def snackbar(content = nil, **props) = snack_bar(content, **props)

#spinkit(**variant) ⇒ Object

spinkit(wave: { color: “red”, size: 50 }) — one variant keyword whose value is the props hash. See flet.dev/docs/controls/spinkit/

Raises:

  • (ArgumentError)


644
645
646
647
648
649
650
651
652
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 644

def spinkit(**variant)
  raise ArgumentError, "spinkit expects exactly one variant, e.g. spinkit(wave: { color: ... })" unless variant.size == 1

  name, props = variant.first
  props ||= {}
  raise ArgumentError, "spinkit #{name} options must be a Hash" unless props.is_a?(Hash)

  build_widget(:"spinkit_#{name}", **props)
end

#stack(children = nil, **props, &block) ⇒ Object



41
42
43
44
45
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 41

def stack(children = nil, **props, &block)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:stack, **mapped, &block)
end


243
244
245
246
247
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 243

def submenu_button(children = nil, **props)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:submenubutton, **mapped)
end


248
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 248

def submenubutton(children = nil, **props) = submenu_button(children, **props)

#switch(**props) ⇒ Object



504
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 504

def switch(**props) = build_widget(:switch, **props)

#tab(label = nil, **props, &block) ⇒ Object



566
567
568
569
570
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 566

def tab(label = nil, **props, &block)
  mapped = props.dup
  mapped[:label] = label unless label.nil?
  build_widget(:tab, **mapped, &block)
end

#tab_bar(tabs = nil, **props, &block) ⇒ Object



572
573
574
575
576
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 572

def tab_bar(tabs = nil, **props, &block)
  mapped = props.dup
  mapped[:tabs] = tabs unless tabs.nil?
  build_widget(:tabbar, **mapped, &block)
end

#tab_bar_view(children = nil, **props, &block) ⇒ Object



578
579
580
581
582
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 578

def tab_bar_view(children = nil, **props, &block)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:tabbarview, **mapped, &block)
end

#tabbar(tabs = nil, **props, &block) ⇒ Object



577
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 577

def tabbar(tabs = nil, **props, &block) = tab_bar(tabs, **props, &block)

#tabbarview(children = nil, **props, &block) ⇒ Object



583
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 583

def tabbarview(children = nil, **props, &block) = tab_bar_view(children, **props, &block)

#tabs(content = nil, **props, &block) ⇒ Object



560
561
562
563
564
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 560

def tabs(content = nil, **props, &block)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:tabs, **mapped, &block)
end

#text(value = nil, **props) ⇒ Object



416
417
418
419
420
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 416

def text(value = nil, **props)
  mapped = props.dup
  mapped[:value] = value unless value.nil?
  build_widget(:text, **mapped)
end

#text_button(content = nil, **props) ⇒ Object



430
431
432
433
434
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 430

def text_button(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:textbutton, **mapped)
end

#text_field(value = nil, **props) ⇒ Object



497
498
499
500
501
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 497

def text_field(value = nil, **props)
  mapped = props.dup
  mapped[:value] = value unless value.nil?
  build_widget(:textfield, **mapped)
end

#text_span(text = nil, **props) ⇒ Object



111
112
113
114
115
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 111

def text_span(text = nil, **props)
  mapped = props.dup
  mapped[:text] = text unless text.nil?
  build_widget(:textspan, **mapped)
end

#textbutton(content = nil, **props) ⇒ Object



435
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 435

def textbutton(content = nil, **props) = text_button(content, **props)

#textfield(**props) ⇒ Object



502
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 502

def textfield(**props) = text_field(**props)

#textspan(text = nil, **props) ⇒ Object



116
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 116

def textspan(text = nil, **props) = text_span(text, **props)

#tile_layer(**props) ⇒ Object



180
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 180

def tile_layer(**props) = build_widget(:tilelayer, **props)

#tilelayer(**props) ⇒ Object



181
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 181

def tilelayer(**props) = tile_layer(**props)

#time_picker(**props) ⇒ Object



386
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 386

def time_picker(**props) = build_widget(:timepicker, **props)

#timepicker(**props) ⇒ Object



387
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 387

def timepicker(**props) = time_picker(**props)

#transparent_pointer(content = nil, **props) ⇒ Object



506
507
508
509
510
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 506

def transparent_pointer(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:transparentpointer, **mapped)
end

#transparentpointer(content = nil, **props) ⇒ Object



511
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 511

def transparentpointer(content = nil, **props) = transparent_pointer(content, **props)

#url_launcher(**props) ⇒ Object



556
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 556

def url_launcher(**props) = build_widget(:url_launcher, **props)

#vertical_divider(**props) ⇒ Object



250
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 250

def vertical_divider(**props) = build_widget(:verticaldivider, **props)

#verticaldivider(**props) ⇒ Object



251
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 251

def verticaldivider(**props) = vertical_divider(**props)

#video(**props) ⇒ Object



640
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 640

def video(**props) = build_widget(:video, **props)

#view(children = nil, **props, &block) ⇒ Object



6
7
8
9
10
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 6

def view(children = nil, **props, &block)
  mapped = props.dup
  mapped[:children] = children unless children.nil?
  build_widget(:view, **mapped, &block)
end

#web_view(**props) ⇒ Object



638
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 638

def web_view(**props) = build_widget(:webview, **props)

#webview(**props) ⇒ Object



639
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 639

def webview(**props) = web_view(**props)

#window_drag_area(content = nil, **props) ⇒ Object



252
253
254
255
256
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 252

def window_drag_area(content = nil, **props)
  mapped = props.dup
  mapped[:content] = content unless content.nil?
  build_widget(:windowdragarea, **mapped)
end

#windowdragarea(content = nil, **props) ⇒ Object



257
# File 'lib/ruflet_ui/ruflet/ui/material_control_methods.rb', line 257

def windowdragarea(content = nil, **props) = window_drag_area(content, **props)