Class: Ruflet::Page

Inherits:
Object
  • Object
show all
Defined in:
lib/ruflet_ui/ruflet/page.rb

Defined Under Namespace

Classes: FlashlightService, ScreenBrightnessService, SharedPreferencesService, WakelockService

Constant Summary collapse

PAGE_PROP_KEYS =
%w[dark_theme fonts route rtl show_semantics_debugger theme theme_mode title vertical_alignment horizontal_alignment scroll].freeze
DIALOG_PROP_KEYS =
%w[dialog snack_bar bottom_sheet].freeze
WIDGET_HELPER_METHODS =
(
  Ruflet::UI::MaterialControlMethods.instance_methods(false) +
  Ruflet::UI::CupertinoControlMethods.instance_methods(false) +
  %i[control widget]
).map(&:to_s).to_set.freeze

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(session_id:, client_details:, sender:) ⇒ Page

Returns a new instance of Page.



152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
# File 'lib/ruflet_ui/ruflet/page.rb', line 152

def initialize(session_id:, client_details:, sender:)
  @session_id = session_id
  @client_details = client_details
  @sender = sender
  @control_index = {}
  @wire_index = {}
  @next_wire_id = 100
  @view_id = 20
  @root_controls = []
  @views = []
  @dialogs = []
  @overlay_container_mounted = false
  @dialogs_container_mounted = false
  @services_container_mounted = false
  @visual_service_controls = {}
  @page_event_handlers = {}
  @view_props = {}
  @page_props = { "route" => (client_details["route"] || "/") }
  @overlay_container = Ruflet::Control.new(
    type: "overlay",
    id: "_overlay",
    controls: []
  )
  @services_container = Ruflet::Control.new(
    type: "service_registry",
    id: "_services",
    "_services": [],
    "_internals": { "uid" => Ruflet::Control.generate_id }
  )
  @dialogs_container = Ruflet::Control.new(
    type: "dialogs",
    id: "_dialogs",
    controls: []
  )
  @invoke_waiters = {}
  @invoke_callbacks = {}
  @invoke_waiters_mutex = Mutex.new
  @shared_preferences_proxy = SharedPreferencesService.new(self)
  @wakelock_proxy = WakelockService.new(self)
  @flashlight_proxy = FlashlightService.new(self)
  @screen_brightness_proxy = ScreenBrightnessService.new(self)
  refresh_overlay_container!
  refresh_services_container!
  refresh_dialogs_container!
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(name, *args, &block) ⇒ Object



1311
1312
1313
1314
1315
1316
1317
1318
1319
1320
1321
1322
1323
1324
1325
1326
1327
1328
1329
1330
1331
1332
1333
1334
# File 'lib/ruflet_ui/ruflet/page.rb', line 1311

def method_missing(name, *args, &block)
  method_name = name.to_s
  prop_name = method_name.delete_suffix("=")

  if method_name.end_with?("=")
    if widget_helper_method?(prop_name)
      raise NoMethodError, "Use `#{prop_name}(...)` as a free widget helper, then attach with `page.add(...)`."
    end
    assign_split_prop(prop_name, normalize_value(prop_name, args.first))
    return args.first
  end

  if args.empty? && !block
    return @page_props[method_name] if @page_props.key?(method_name)
    return @view_props[method_name] if @view_props.key?(method_name)
    return instance_variable_get("@#{method_name}") if DIALOG_PROP_KEYS.include?(method_name)
  end

  if widget_helper_method?(name)
    raise NoMethodError, "Use `#{name}(...)` as a free widget helper, then attach with `page.add(...)`."
  end

  super
end

Instance Attribute Details

#client_detailsObject (readonly)

Returns the value of attribute client_details.



150
151
152
# File 'lib/ruflet_ui/ruflet/page.rb', line 150

def client_details
  @client_details
end

#session_idObject (readonly)

Returns the value of attribute session_id.



150
151
152
# File 'lib/ruflet_ui/ruflet/page.rb', line 150

def session_id
  @session_id
end

#viewsObject

Returns the value of attribute views.



150
151
152
# File 'lib/ruflet_ui/ruflet/page.rb', line 150

def views
  @views
end

Instance Method Details

#accelerometer(**props) ⇒ Object



965
966
967
# File 'lib/ruflet_ui/ruflet/page.rb', line 965

def accelerometer(**props)
  service(:accelerometer, **props)
end

#add(*controls, appbar: nil, bottom_appbar: nil, floating_action_button: nil, navigation_bar: nil, dialog: nil, snack_bar: nil, bottom_sheet: nil) ⇒ Object



283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
# File 'lib/ruflet_ui/ruflet/page.rb', line 283

def add(*controls, appbar: nil, bottom_appbar: nil, floating_action_button: nil, navigation_bar: nil, dialog: nil, snack_bar: nil, bottom_sheet: nil)
  controls = controls.flatten
  visited = Set.new
  controls.each { |c| register_control_tree(c, visited) }
  @root_controls = controls

  update_view_slot("appbar", appbar)
  update_view_slot("bottom_appbar", bottom_appbar)
  update_view_slot("floating_action_button", floating_action_button)
  update_view_slot("navigation_bar", navigation_bar)
  @dialog = dialog if dialog
  @snack_bar = snack_bar if snack_bar
  @bottom_sheet = bottom_sheet if bottom_sheet

  refresh_dialogs_container!
  @view_props.each_value { |value| register_embedded_value(value, visited) }

  send_view_patch

  self
end

#add_service(*value) ⇒ Object



365
366
367
368
369
370
# File 'lib/ruflet_ui/ruflet/page.rb', line 365

def add_service(*value)
  @services_container.props["_services"] = services + value.flatten.compact
  refresh_services_container!
  push_services_update!
  self
end

#appbar=(value) ⇒ Object



466
467
468
# File 'lib/ruflet_ui/ruflet/page.rb', line 466

def appbar=(value)
  @view_props["appbar"] = value
end

#apply_client_update(control_or_id, props) ⇒ Object



1263
1264
1265
1266
1267
1268
1269
1270
1271
# File 'lib/ruflet_ui/ruflet/page.rb', line 1263

def apply_client_update(control_or_id, props)
  control = resolve_control(control_or_id)
  return self unless control

  patch = normalize_props(props || {})
  patch.each { |k, v| control.props[k] = v }

  self
end

#audio(**props) ⇒ Object



345
346
347
# File 'lib/ruflet_ui/ruflet/page.rb', line 345

def audio(**props)
  service(:audio, **props)
end

#audio_recorder(**props) ⇒ Object



349
350
351
# File 'lib/ruflet_ui/ruflet/page.rb', line 349

def audio_recorder(**props)
  service(:audio_recorder, **props)
end

#barometer(**props) ⇒ Object



981
982
983
# File 'lib/ruflet_ui/ruflet/page.rb', line 981

def barometer(**props)
  service(:barometer, **props)
end

#battery(**props) ⇒ Object



997
998
999
# File 'lib/ruflet_ui/ruflet/page.rb', line 997

def battery(**props)
  service(:battery, **props)
end

#battery_save_mode?(timeout: nil, on_result: nil) ⇒ Boolean

Returns:

  • (Boolean)


961
962
963
# File 'lib/ruflet_ui/ruflet/page.rb', line 961

def battery_save_mode?(timeout: nil, on_result: nil)
  is_in_battery_save_mode(timeout: timeout, on_result: on_result)
end

#bgcolorObject



239
240
241
# File 'lib/ruflet_ui/ruflet/page.rb', line 239

def bgcolor
  @view_props["bgcolor"]
end

#bgcolor=(value) ⇒ Object



243
244
245
# File 'lib/ruflet_ui/ruflet/page.rb', line 243

def bgcolor=(value)
  @view_props["bgcolor"] = normalize_value("bgcolor", value)
end

#bottom_appbar=(value) ⇒ Object



470
471
472
# File 'lib/ruflet_ui/ruflet/page.rb', line 470

def bottom_appbar=(value)
  @view_props["bottom_appbar"] = value
end

#bottom_sheet=(value) ⇒ Object



540
541
542
543
# File 'lib/ruflet_ui/ruflet/page.rb', line 540

def bottom_sheet=(value)
  @bottom_sheet = value
  refresh_dialogs_container!
end

#bottomappbar=(value) ⇒ Object



474
475
476
# File 'lib/ruflet_ui/ruflet/page.rb', line 474

def bottomappbar=(value)
  self.bottom_appbar = value
end

#bottomsheet=(value) ⇒ Object



545
546
547
# File 'lib/ruflet_ui/ruflet/page.rb', line 545

def bottomsheet=(value)
  self.bottom_sheet = value
end

#browser_context_menu(**props) ⇒ Object



353
354
355
# File 'lib/ruflet_ui/ruflet/page.rb', line 353

def browser_context_menu(**props)
  service(:browser_context_menu, **props)
end

#camera(**props) ⇒ Object



1025
1026
1027
# File 'lib/ruflet_ui/ruflet/page.rb', line 1025

def camera(**props)
  service(:camera, **props)
end

#can_launch_url(url, timeout: 10) ⇒ Object



620
621
622
623
# File 'lib/ruflet_ui/ruflet/page.rb', line 620

def can_launch_url(url, timeout: 10)
  url_launcher = ensure_url_launcher_service
  invoke(url_launcher, "can_launch_url", args: { "url" => url }, timeout: timeout)
end

#center_window(timeout: 10, on_result: nil) ⇒ Object



741
742
743
# File 'lib/ruflet_ui/ruflet/page.rb', line 741

def center_window(timeout: 10, on_result: nil)
  invoke_window("center", timeout: timeout, on_result: on_result)
end

#clipboard(**props) ⇒ Object



1005
1006
1007
# File 'lib/ruflet_ui/ruflet/page.rb', line 1005

def clipboard(**props)
  service(:clipboard, **props)
end

#close_dialog(dialog_control) ⇒ Object



1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
# File 'lib/ruflet_ui/ruflet/page.rb', line 1197

def close_dialog(dialog_control)
  return self unless dialog_control

  dialog_control.props["open"] = false
  @dialog = nil if @dialog.equal?(dialog_control)
  remove_dialog_tracking(dialog_control)
  refresh_dialogs_container!
  # Patch the dialogs container in place. Forcing a full view re-render
  # here would remount the whole overlay — fatal while another dialog
  # (e.g. the form behind a nested picker) is still open. The empty case
  # is handled inside push_dialogs_update!.
  push_dialogs_update!
  self
end

#close_drawer(timeout: 10, on_result: nil) ⇒ Object



505
506
507
508
# File 'lib/ruflet_ui/ruflet/page.rb', line 505

def close_drawer(timeout: 10, on_result: nil)
  invoke_current_view("close_drawer", timeout: timeout, on_result: on_result)
  self
end

#close_end_drawer(timeout: 10, on_result: nil) ⇒ Object



517
518
519
520
# File 'lib/ruflet_ui/ruflet/page.rb', line 517

def close_end_drawer(timeout: 10, on_result: nil)
  invoke_current_view("close_end_drawer", timeout: timeout, on_result: on_result)
  self
end

#close_in_app_web_view(timeout: 10, on_result: nil) ⇒ Object



625
626
627
628
# File 'lib/ruflet_ui/ruflet/page.rb', line 625

def close_in_app_web_view(timeout: 10, on_result: nil)
  url_launcher = ensure_url_launcher_service
  invoke(url_launcher, "close_in_app_web_view", timeout: timeout, on_result: on_result)
end

#close_window(timeout: 10, on_result: nil) ⇒ Object



745
746
747
# File 'lib/ruflet_ui/ruflet/page.rb', line 745

def close_window(timeout: 10, on_result: nil)
  invoke_window("close", timeout: timeout, on_result: on_result)
end

#connectivity(**props) ⇒ Object



1001
1002
1003
# File 'lib/ruflet_ui/ruflet/page.rb', line 1001

def connectivity(**props)
  service(:connectivity, **props)
end

#destroy_window(timeout: 10, on_result: nil) ⇒ Object



749
750
751
# File 'lib/ruflet_ui/ruflet/page.rb', line 749

def destroy_window(timeout: 10, on_result: nil)
  invoke_window("destroy", timeout: timeout, on_result: on_result)
end

#dialogObject



522
# File 'lib/ruflet_ui/ruflet/page.rb', line 522

def dialog = @dialog

#dialog=(value) ⇒ Object



524
525
526
527
528
# File 'lib/ruflet_ui/ruflet/page.rb', line 524

def dialog=(value)
  @dialog = value
  refresh_dialogs_container!
  push_dialogs_update! if @dialogs_container_mounted
end

#disable_browser_context_menu(timeout: 10, on_result: nil) ⇒ Object



725
726
727
# File 'lib/ruflet_ui/ruflet/page.rb', line 725

def disable_browser_context_menu(timeout: 10, on_result: nil)
  invoke_browser_context_menu("disable_menu", timeout: timeout, on_result: on_result)
end

#dispatch_event(target:, name:, data:) ⇒ Object



1273
1274
1275
1276
1277
1278
1279
1280
1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
1291
1292
1293
1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
1306
1307
1308
1309
# File 'lib/ruflet_ui/ruflet/page.rb', line 1273

def dispatch_event(target:, name:, data:)
  if page_control_target?(target)
    if name.to_s == "route_change"
      route_from_event = extract_route(data)
      # Dialogs (including pickers) belong to the view that opened them.
      # Navigating away must dismiss them, or they ghost onto the next
      # view — the picker that "reappears after going home".
      dismiss_tracked_dialogs! if route_from_event && route_from_event != @page_props["route"]
      @page_props["route"] = route_from_event if route_from_event
    elsif name.to_s == "resize"
      # The client reports the live page size via the "resize" event. Store
      # it so `page.width`/`page.height` reflect the real viewport — without
      # this, responsive layouts collapse on clients (e.g. embedded/iOS)
      # that don't know their size at the initial handshake.
      store_reported_page_size(data)
    end
    dispatch_page_event(name: name, data: data)
    return
  end

  control = @wire_index[target.to_i] || @control_index[target.to_s]
  return unless control

  event = Ruflet::Event.new(name: name, target: target, raw_data: data, page: self, control: control)
  apply_event_value_to_control(control, event) if %w[change select select_change].include?(name.to_s)
  # Material/Cupertino pickers dismiss themselves on the client once a
  # value is confirmed, but only send a value event — never a close. Mark
  # the dialog closed here so show_dialog can reopen it next time.
  mark_picker_dialog_closed(control, name)
  if dialog_close_event?(control, name) && remove_dialog_tracking(control)
    # Patch the container in place; never force a full view re-render that
    # would remount a still-open parent dialog (the nested-picker case).
    push_dialogs_update!
  end

  control.emit(name, event)
end

#drag(finder_id, offset, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



836
837
838
839
840
841
842
843
844
845
846
847
# File 'lib/ruflet_ui/ruflet/page.rb', line 836

def drag(finder_id, offset, finder_index: nil, timeout: 10, on_result: nil)
  invoke_tester(
    "drag",
    args: compact_service_args(
      "finder_id" => finder_id,
      "finder_index" => finder_index,
      "offset" => offset
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#drag_from(start, offset, timeout: 10, on_result: nil) ⇒ Object



849
850
851
852
853
854
855
856
# File 'lib/ruflet_ui/ruflet/page.rb', line 849

def drag_from(start, offset, timeout: 10, on_result: nil)
  invoke_tester(
    "drag_from",
    args: compact_service_args("start" => start, "offset" => offset),
    timeout: timeout,
    on_result: on_result
  )
end

#drawerObject



482
483
484
# File 'lib/ruflet_ui/ruflet/page.rb', line 482

def drawer
  @view_props["drawer"]
end

#drawer=(value) ⇒ Object



486
487
488
# File 'lib/ruflet_ui/ruflet/page.rb', line 486

def drawer=(value)
  @view_props["drawer"] = value
end

#enable_browser_context_menu(timeout: 10, on_result: nil) ⇒ Object



729
730
731
# File 'lib/ruflet_ui/ruflet/page.rb', line 729

def enable_browser_context_menu(timeout: 10, on_result: nil)
  invoke_browser_context_menu("enable_menu", timeout: timeout, on_result: on_result)
end

#end_drawerObject



490
491
492
# File 'lib/ruflet_ui/ruflet/page.rb', line 490

def end_drawer
  @view_props["end_drawer"]
end

#end_drawer=(value) ⇒ Object



494
495
496
# File 'lib/ruflet_ui/ruflet/page.rb', line 494

def end_drawer=(value)
  @view_props["end_drawer"] = value
end

#enter_text(finder_id, text, options = nil, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



863
864
865
866
867
868
869
870
871
872
873
874
875
# File 'lib/ruflet_ui/ruflet/page.rb', line 863

def enter_text(finder_id, text, options = nil, finder_index: nil, timeout: 10, on_result: nil)
  finder_index = options[:finder_index] || options["finder_index"] if options.is_a?(Hash) && finder_index.nil?
  invoke_tester(
    "enter_text",
    args: compact_service_args(
      "finder_id" => finder_id,
      "finder_index" => finder_index,
      "text" => text
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#file_picker(**props) ⇒ Object



1009
1010
1011
# File 'lib/ruflet_ui/ruflet/page.rb', line 1009

def file_picker(**props)
  service(:file_picker, **props)
end

#find_by_icon(icon, timeout: 10, on_result: nil) ⇒ Object



792
793
794
# File 'lib/ruflet_ui/ruflet/page.rb', line 792

def find_by_icon(icon, timeout: 10, on_result: nil)
  invoke_tester("find_by_icon", args: { "icon" => normalize_service_value(icon) }, timeout: timeout, on_result: on_result)
end

#find_by_key(key, timeout: 10, on_result: nil) ⇒ Object



784
785
786
# File 'lib/ruflet_ui/ruflet/page.rb', line 784

def find_by_key(key, timeout: 10, on_result: nil)
  invoke_tester("find_by_key", args: { "key" => key }, timeout: timeout, on_result: on_result)
end

#find_by_text(text, timeout: 10, on_result: nil) ⇒ Object



776
777
778
# File 'lib/ruflet_ui/ruflet/page.rb', line 776

def find_by_text(text, timeout: 10, on_result: nil)
  invoke_tester("find_by_text", args: { "text" => text }, timeout: timeout, on_result: on_result)
end

#find_by_text_containing(pattern, timeout: 10, on_result: nil) ⇒ Object



780
781
782
# File 'lib/ruflet_ui/ruflet/page.rb', line 780

def find_by_text_containing(pattern, timeout: 10, on_result: nil)
  invoke_tester("find_by_text_containing", args: { "pattern" => pattern }, timeout: timeout, on_result: on_result)
end

#find_by_tooltip(value, timeout: 10, on_result: nil) ⇒ Object



788
789
790
# File 'lib/ruflet_ui/ruflet/page.rb', line 788

def find_by_tooltip(value, timeout: 10, on_result: nil)
  invoke_tester("find_by_tooltip", args: { "value" => value }, timeout: timeout, on_result: on_result)
end

#flashlight(**props) ⇒ Object



333
334
335
336
337
# File 'lib/ruflet_ui/ruflet/page.rb', line 333

def flashlight(**props)
  return service(:flashlight, **props) unless props.empty?

  @flashlight_proxy
end

#floating_action_button=(value) ⇒ Object



478
479
480
# File 'lib/ruflet_ui/ruflet/page.rb', line 478

def floating_action_button=(value)
  @view_props["floating_action_button"] = value
end

#geolocator(**props) ⇒ Object



1033
1034
1035
# File 'lib/ruflet_ui/ruflet/page.rb', line 1033

def geolocator(**props)
  service(:geolocator, **props)
end

#get_application_cache_directory(timeout: nil, on_result: nil) ⇒ Object



1045
1046
1047
# File 'lib/ruflet_ui/ruflet/page.rb', line 1045

def get_application_cache_directory(timeout: nil, on_result: nil)
  invoke_storage_paths("get_application_cache_directory", timeout: timeout, on_result: on_result)
end

#get_application_documents_directory(timeout: nil, on_result: nil) ⇒ Object



1049
1050
1051
# File 'lib/ruflet_ui/ruflet/page.rb', line 1049

def get_application_documents_directory(timeout: nil, on_result: nil)
  invoke_storage_paths("get_application_documents_directory", timeout: timeout, on_result: on_result)
end

#get_application_support_directory(timeout: nil, on_result: nil) ⇒ Object



1053
1054
1055
# File 'lib/ruflet_ui/ruflet/page.rb', line 1053

def get_application_support_directory(timeout: nil, on_result: nil)
  invoke_storage_paths("get_application_support_directory", timeout: timeout, on_result: on_result)
end

#get_battery_level(timeout: nil, on_result: nil) ⇒ Object



949
950
951
# File 'lib/ruflet_ui/ruflet/page.rb', line 949

def get_battery_level(timeout: nil, on_result: nil)
  invoke_battery_method("get_battery_level", timeout: timeout, on_result: on_result)
end

#get_battery_state(timeout: nil, on_result: nil) ⇒ Object



953
954
955
# File 'lib/ruflet_ui/ruflet/page.rb', line 953

def get_battery_state(timeout: nil, on_result: nil)
  invoke_battery_method("get_battery_state", timeout: timeout, on_result: on_result)
end

#get_clipboard(timeout: nil, on_result: nil) ⇒ Object



915
916
917
# File 'lib/ruflet_ui/ruflet/page.rb', line 915

def get_clipboard(timeout: nil, on_result: nil)
  invoke_clipboard_method("get", timeout: timeout, on_result: on_result)
end

#get_clipboard_files(timeout: nil, on_result: nil) ⇒ Object



928
929
930
# File 'lib/ruflet_ui/ruflet/page.rb', line 928

def get_clipboard_files(timeout: nil, on_result: nil)
  invoke_clipboard_method("get_files", timeout: timeout, on_result: on_result)
end

#get_clipboard_image(timeout: nil, on_result: nil) ⇒ Object



941
942
943
# File 'lib/ruflet_ui/ruflet/page.rb', line 941

def get_clipboard_image(timeout: nil, on_result: nil)
  invoke_clipboard_method("get_image", timeout: timeout, on_result: on_result)
end

#get_connectivity(timeout: nil, on_result: nil) ⇒ Object



945
946
947
# File 'lib/ruflet_ui/ruflet/page.rb', line 945

def get_connectivity(timeout: nil, on_result: nil)
  invoke_connectivity_method("get_connectivity", timeout: timeout, on_result: on_result)
end

#get_console_log_filename(timeout: nil, on_result: nil) ⇒ Object



1081
1082
1083
# File 'lib/ruflet_ui/ruflet/page.rb', line 1081

def get_console_log_filename(timeout: nil, on_result: nil)
  invoke_storage_paths("get_console_log_filename", timeout: timeout, on_result: on_result)
end

#get_directory_path(dialog_title: nil, initial_directory: nil, timeout: nil, on_result: nil) ⇒ Object



700
701
702
703
704
705
706
707
708
709
710
# File 'lib/ruflet_ui/ruflet/page.rb', line 700

def get_directory_path(dialog_title: nil, initial_directory: nil, timeout: nil, on_result: nil)
  invoke_file_picker(
    "get_directory_path",
    compact_service_args(
      "dialog_title" => dialog_title,
      "initial_directory" => initial_directory
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#get_downloads_directory(timeout: nil, on_result: nil) ⇒ Object



1057
1058
1059
# File 'lib/ruflet_ui/ruflet/page.rb', line 1057

def get_downloads_directory(timeout: nil, on_result: nil)
  invoke_storage_paths("get_downloads_directory", timeout: timeout, on_result: on_result)
end

#get_external_cache_directories(timeout: nil, on_result: nil) ⇒ Object



1061
1062
1063
# File 'lib/ruflet_ui/ruflet/page.rb', line 1061

def get_external_cache_directories(timeout: nil, on_result: nil)
  invoke_storage_paths("get_external_cache_directories", timeout: timeout, on_result: on_result)
end

#get_external_storage_directories(timeout: nil, on_result: nil) ⇒ Object



1065
1066
1067
# File 'lib/ruflet_ui/ruflet/page.rb', line 1065

def get_external_storage_directories(timeout: nil, on_result: nil)
  invoke_storage_paths("get_external_storage_directories", timeout: timeout, on_result: on_result)
end

#get_external_storage_directory(timeout: nil, on_result: nil) ⇒ Object



1073
1074
1075
# File 'lib/ruflet_ui/ruflet/page.rb', line 1073

def get_external_storage_directory(timeout: nil, on_result: nil)
  invoke_storage_paths("get_external_storage_directory", timeout: timeout, on_result: on_result)
end

#get_library_directory(timeout: nil, on_result: nil) ⇒ Object



1069
1070
1071
# File 'lib/ruflet_ui/ruflet/page.rb', line 1069

def get_library_directory(timeout: nil, on_result: nil)
  invoke_storage_paths("get_library_directory", timeout: timeout, on_result: on_result)
end

#get_temporary_directory(timeout: nil, on_result: nil) ⇒ Object



1077
1078
1079
# File 'lib/ruflet_ui/ruflet/page.rb', line 1077

def get_temporary_directory(timeout: nil, on_result: nil)
  invoke_storage_paths("get_temporary_directory", timeout: timeout, on_result: on_result)
end

#go(route, **query_params) ⇒ Object



424
425
426
427
428
429
# File 'lib/ruflet_ui/ruflet/page.rb', line 424

def go(route, **query_params)
  @page_props["route"] = build_route(route, query_params)
  dispatch_page_event(name: "route_change", data: @page_props["route"])
  send_view_patch
  self
end

#gyroscope(**props) ⇒ Object



969
970
971
# File 'lib/ruflet_ui/ruflet/page.rb', line 969

def gyroscope(**props)
  service(:gyroscope, **props)
end

#handle_invoke_method_result(payload) ⇒ Object



1170
1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
1184
1185
1186
# File 'lib/ruflet_ui/ruflet/page.rb', line 1170

def handle_invoke_method_result(payload)
  call_id = payload["call_id"].to_s
  waiter = @invoke_waiters_mutex.synchronize { @invoke_waiters[call_id] }
  if waiter
    waiter << payload
    return true
  end

  callback = @invoke_waiters_mutex.synchronize { @invoke_callbacks.delete(call_id) }
  return false unless callback

  callback.call(payload["result"], payload["error"])
  true
rescue StandardError => e
  Kernel.warn("invoke callback error: #{e.class}: #{e.message}")
  false
end

#haptic_feedback(**props) ⇒ Object



1029
1030
1031
# File 'lib/ruflet_ui/ruflet/page.rb', line 1029

def haptic_feedback(**props)
  service(:haptic_feedback, **props)
end

#heavy_impact(timeout: 10, on_result: nil) ⇒ Object



886
887
888
# File 'lib/ruflet_ui/ruflet/page.rb', line 886

def heavy_impact(timeout: 10, on_result: nil)
  invoke_haptic_feedback("heavy_impact", timeout: timeout, on_result: on_result)
end

#heightObject



255
256
257
# File 'lib/ruflet_ui/ruflet/page.rb', line 255

def height
  client_reported_prop("height")
end

#horizontal_alignmentObject



229
230
231
# File 'lib/ruflet_ui/ruflet/page.rb', line 229

def horizontal_alignment
  @page_props["horizontal_alignment"] || @view_props["horizontal_alignment"]
end

#horizontal_alignment=(value) ⇒ Object



233
234
235
236
237
# File 'lib/ruflet_ui/ruflet/page.rb', line 233

def horizontal_alignment=(value)
  v = normalize_value("horizontal_alignment", value)
  @page_props["horizontal_alignment"] = v
  @view_props["horizontal_alignment"] = v
end

#invoke(control_or_id, method_name, args: nil, timeout: 10, on_result: nil) ⇒ Object



563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
# File 'lib/ruflet_ui/ruflet/page.rb', line 563

def invoke(control_or_id, method_name, args: nil, timeout: 10, on_result: nil)
  control_id =
    if page_control_target?(control_or_id)
      1
    else
      control = resolve_control(control_or_id)
      return nil unless control
      control.wire_id
    end

  call_id = "call_#{Ruflet::Control.generate_id}"
  if on_result.respond_to?(:call)
    @invoke_waiters_mutex.synchronize { @invoke_callbacks[call_id] = on_result }
    if embedded_async_timeout_available? && !timeout.nil?
      Thread.new(call_id, timeout.to_f) do |pending_call_id, invoke_timeout|
        sleep([invoke_timeout, 0.0].max + 0.1)
        callback = @invoke_waiters_mutex.synchronize { @invoke_callbacks.delete(pending_call_id) }
        callback&.call(nil, "execution expired")
      rescue StandardError => e
        Kernel.warn("invoke timeout callback error: #{e.class}: #{e.message}")
      end
    end
  end
  payload = {
    "control_id" => control_id,
    "call_id" => call_id,
    "name" => method_name.to_s,
    "args" => args
  }
  payload["timeout"] = timeout unless timeout.nil?
  send_message(Protocol::ACTIONS[:invoke_control_method], payload)

  call_id
end

#invoke_sync(control_or_id, method_name, args: nil, timeout: 10) ⇒ Object

Synchronous invoke for controls/services that must return a value before continuing (e.g. picker selection, camera discovery/init).



600
601
602
# File 'lib/ruflet_ui/ruflet/page.rb', line 600

def invoke_sync(control_or_id, method_name, args: nil, timeout: 10)
  invoke_and_wait(control_or_id, method_name, args: args, timeout: timeout)
end

#is_in_battery_save_mode(timeout: nil, on_result: nil) ⇒ Object



957
958
959
# File 'lib/ruflet_ui/ruflet/page.rb', line 957

def is_in_battery_save_mode(timeout: nil, on_result: nil)
  invoke_battery_method("is_in_battery_save_mode", timeout: timeout, on_result: on_result)
end

#launch_url(url, mode: nil, web_view_configuration: nil, browser_configuration: nil, web_only_window_name: nil, timeout: 10, on_result: nil) ⇒ Object



604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
# File 'lib/ruflet_ui/ruflet/page.rb', line 604

def launch_url(url, mode: nil, web_view_configuration: nil, browser_configuration: nil, web_only_window_name: nil, timeout: 10, on_result: nil)
  url_launcher = ensure_url_launcher_service
  args = { "url" => url }
  args["mode"] = mode unless mode.nil?
  args["web_view_configuration"] = web_view_configuration unless web_view_configuration.nil?
  args["browser_configuration"] = browser_configuration unless browser_configuration.nil?
  args["web_only_window_name"] = web_only_window_name unless web_only_window_name.nil?
  invoke(
    url_launcher,
    "launch_url",
    args: args,
    timeout: timeout,
    on_result: on_result
  )
end

#light_impact(timeout: 10, on_result: nil) ⇒ Object



894
895
896
# File 'lib/ruflet_ui/ruflet/page.rb', line 894

def light_impact(timeout: 10, on_result: nil)
  invoke_haptic_feedback("light_impact", timeout: timeout, on_result: on_result)
end

#long_press(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



858
859
860
861
# File 'lib/ruflet_ui/ruflet/page.rb', line 858

def long_press(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil)
  finder_index = options[:finder_index] || options["finder_index"] if options.is_a?(Hash) && finder_index.nil?
  invoke_tester_finder("long_press", finder_id, finder_index: finder_index, timeout: timeout, on_result: on_result)
end

#magnetometer(**props) ⇒ Object



977
978
979
# File 'lib/ruflet_ui/ruflet/page.rb', line 977

def magnetometer(**props)
  service(:magnetometer, **props)
end

#mediaObject



279
280
281
# File 'lib/ruflet_ui/ruflet/page.rb', line 279

def media
  client_reported_prop("media")
end

#medium_impact(timeout: 10, on_result: nil) ⇒ Object



890
891
892
# File 'lib/ruflet_ui/ruflet/page.rb', line 890

def medium_impact(timeout: 10, on_result: nil)
  invoke_haptic_feedback("medium_impact", timeout: timeout, on_result: on_result)
end

#mount(&block) ⇒ Object



460
461
462
463
464
# File 'lib/ruflet_ui/ruflet/page.rb', line 460

def mount(&block)
  builder = WidgetBuilder.new
  builder.instance_eval(&block)
  add(*builder.children)
end

#mouse_click(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



805
806
807
808
# File 'lib/ruflet_ui/ruflet/page.rb', line 805

def mouse_click(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil)
  finder_index = options[:finder_index] || options["finder_index"] if options.is_a?(Hash) && finder_index.nil?
  invoke_tester_finder("mouse_click", finder_id, finder_index: finder_index, timeout: timeout, on_result: on_result)
end

#mouse_click_at(offset = nil, timeout: 10, on_result: nil) ⇒ Object



824
825
826
# File 'lib/ruflet_ui/ruflet/page.rb', line 824

def mouse_click_at(offset = nil, timeout: 10, on_result: nil)
  invoke_tester_at("mouse_click_at", offset, timeout: timeout, on_result: on_result)
end

#mouse_double_click(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



810
811
812
813
# File 'lib/ruflet_ui/ruflet/page.rb', line 810

def mouse_double_click(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil)
  finder_index = options[:finder_index] || options["finder_index"] if options.is_a?(Hash) && finder_index.nil?
  invoke_tester_finder("mouse_double_click", finder_id, finder_index: finder_index, timeout: timeout, on_result: on_result)
end

#mouse_double_click_at(offset = nil, timeout: 10, on_result: nil) ⇒ Object



828
829
830
# File 'lib/ruflet_ui/ruflet/page.rb', line 828

def mouse_double_click_at(offset = nil, timeout: 10, on_result: nil)
  invoke_tester_at("mouse_double_click_at", offset, timeout: timeout, on_result: on_result)
end

#mouse_hover(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



877
878
879
880
# File 'lib/ruflet_ui/ruflet/page.rb', line 877

def mouse_hover(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil)
  finder_index = options[:finder_index] || options["finder_index"] if options.is_a?(Hash) && finder_index.nil?
  invoke_tester_finder("mouse_hover", finder_id, finder_index: finder_index, timeout: timeout, on_result: on_result)
end


431
432
433
# File 'lib/ruflet_ui/ruflet/page.rb', line 431

def navigate(route, **query_params)
  go(route, **query_params)
end

#on(event_name, &block) ⇒ Object



455
456
457
458
# File 'lib/ruflet_ui/ruflet/page.rb', line 455

def on(event_name, &block)
  @page_event_handlers[event_name.to_s.sub(/\Aon_/, "")] = block
  self
end

#on_resize=(handler) ⇒ Object



451
452
453
# File 'lib/ruflet_ui/ruflet/page.rb', line 451

def on_resize=(handler)
  @page_event_handlers["resize"] = handler
end

#on_route_change=(handler) ⇒ Object



443
444
445
# File 'lib/ruflet_ui/ruflet/page.rb', line 443

def on_route_change=(handler)
  @page_event_handlers["route_change"] = handler
end

#on_view_pop=(handler) ⇒ Object



447
448
449
# File 'lib/ruflet_ui/ruflet/page.rb', line 447

def on_view_pop=(handler)
  @page_event_handlers["view_pop"] = handler
end

#open_window(url, title: nil, width: nil, height: nil, timeout: 10, on_result: nil) ⇒ Object



630
631
632
633
634
635
636
637
# File 'lib/ruflet_ui/ruflet/page.rb', line 630

def open_window(url, title: nil, width: nil, height: nil, timeout: 10, on_result: nil)
  url_launcher = ensure_url_launcher_service
  args = { "url" => url }
  args["title"] = title unless title.nil?
  args["width"] = width unless width.nil?
  args["height"] = height unless height.nil?
  invoke(url_launcher, "open_window", args: args, timeout: timeout, on_result: on_result)
end

#patch_page(control_id, **props) ⇒ Object



1259
1260
1261
# File 'lib/ruflet_ui/ruflet/page.rb', line 1259

def patch_page(control_id, **props)
  update(control_id, **props)
end

#permission_handler(**props) ⇒ Object



1037
1038
1039
# File 'lib/ruflet_ui/ruflet/page.rb', line 1037

def permission_handler(**props)
  service(:permission_handler, **props)
end

#pick_files(dialog_title: nil, initial_directory: nil, file_type: "any", allowed_extensions: nil, allow_multiple: false, with_data: false, timeout: nil, on_result: nil) ⇒ Object

File picker helpers: create an ephemeral service, invoke method, and dispose it.



650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
# File 'lib/ruflet_ui/ruflet/page.rb', line 650

def pick_files(
  dialog_title: nil,
  initial_directory: nil,
  file_type: "any",
  allowed_extensions: nil,
  allow_multiple: false,
  with_data: false,
  timeout: nil,
  on_result: nil
)
  invoke_file_picker(
    "pick_files",
    compact_service_args(
      "dialog_title" => dialog_title,
      "initial_directory" => initial_directory,
      "file_type" => file_type,
      "allowed_extensions" => allowed_extensions,
      "allow_multiple" => allow_multiple,
      "with_data" => with_data
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#platformObject



259
260
261
# File 'lib/ruflet_ui/ruflet/page.rb', line 259

def platform
  client_reported_prop("platform")
end

#platform_brightnessObject



263
264
265
# File 'lib/ruflet_ui/ruflet/page.rb', line 263

def platform_brightness
  client_reported_prop("platform_brightness")
end

#pop_dialogObject



1188
1189
1190
1191
1192
1193
1194
1195
# File 'lib/ruflet_ui/ruflet/page.rb', line 1188

def pop_dialog
  dialog_control = latest_open_dialog
  return nil unless dialog_control

  dialog_control.props["open"] = false
  update(dialog_control, open: false)
  dialog_control
end

#push_route(route, **query_params) ⇒ Object



435
436
437
# File 'lib/ruflet_ui/ruflet/page.rb', line 435

def push_route(route, **query_params)
  go(route, **query_params)
end

#pwaObject



271
272
273
# File 'lib/ruflet_ui/ruflet/page.rb', line 271

def pwa
  client_reported_prop("pwa")
end

#queryObject



439
440
441
# File 'lib/ruflet_ui/ruflet/page.rb', line 439

def query
  parse_query(route)
end

#remove_service(*value) ⇒ Object



372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
# File 'lib/ruflet_ui/ruflet/page.rb', line 372

def remove_service(*value)
  targets = value.flatten.compact
  return self if targets.empty?

  @services_container.props["_services"] = services.reject do |service|
    targets.any? do |target|
      case target
      when Control
        service.equal?(target) || (!target.id.nil? && service.id.to_s == target.id.to_s)
      else
        needle = target.to_s
        service.id.to_s == needle || service.type.to_s.downcase == needle.downcase
      end
    end
  end

  refresh_services_container!
  push_services_update!
  self
end

#respond_to_missing?(name, include_private = false) ⇒ Boolean

Returns:

  • (Boolean)


1336
1337
1338
1339
1340
1341
1342
1343
1344
1345
1346
# File 'lib/ruflet_ui/ruflet/page.rb', line 1336

def respond_to_missing?(name, include_private = false)
  method_name = name.to_s
  prop_name = method_name.delete_suffix("=")
  widget_helper_method?(name) ||
    widget_helper_method?(prop_name) ||
    method_name.end_with?("=") ||
    @page_props.key?(method_name) ||
    @view_props.key?(method_name) ||
    DIALOG_PROP_KEYS.include?(method_name) ||
    super
end

#right_mouse_click(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



815
816
817
818
# File 'lib/ruflet_ui/ruflet/page.rb', line 815

def right_mouse_click(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil)
  finder_index = options[:finder_index] || options["finder_index"] if options.is_a?(Hash) && finder_index.nil?
  invoke_tester_finder("right_mouse_click", finder_id, finder_index: finder_index, timeout: timeout, on_result: on_result)
end

#right_mouse_click_at(offset = nil, timeout: 10, on_result: nil) ⇒ Object



832
833
834
# File 'lib/ruflet_ui/ruflet/page.rb', line 832

def right_mouse_click_at(offset = nil, timeout: 10, on_result: nil)
  invoke_tester_at("right_mouse_click_at", offset, timeout: timeout, on_result: on_result)
end

#routeObject



211
212
213
# File 'lib/ruflet_ui/ruflet/page.rb', line 211

def route
  @page_props["route"]
end

#route=(value) ⇒ Object



215
216
217
# File 'lib/ruflet_ui/ruflet/page.rb', line 215

def route=(value)
  @page_props["route"] = value
end

#save_file(dialog_title: nil, file_name: nil, initial_directory: nil, file_type: "any", allowed_extensions: nil, src_bytes: nil, timeout: nil, on_result: nil) ⇒ Object



675
676
677
678
679
680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
695
696
697
698
# File 'lib/ruflet_ui/ruflet/page.rb', line 675

def save_file(
  dialog_title: nil,
  file_name: nil,
  initial_directory: nil,
  file_type: "any",
  allowed_extensions: nil,
  src_bytes: nil,
  timeout: nil,
  on_result: nil
)
  invoke_file_picker(
    "save_file",
    compact_service_args(
      "dialog_title" => dialog_title,
      "file_name" => file_name,
      "initial_directory" => initial_directory,
      "file_type" => file_type,
      "allowed_extensions" => allowed_extensions,
      "src_bytes" => src_bytes
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#screen_brightness(**props) ⇒ Object



339
340
341
342
343
# File 'lib/ruflet_ui/ruflet/page.rb', line 339

def screen_brightness(**props)
  return service(:screen_brightness, **props) unless props.empty?

  @screen_brightness_proxy
end

#screenshot(**props) ⇒ Object



993
994
995
# File 'lib/ruflet_ui/ruflet/page.rb', line 993

def screenshot(**props)
  service(:screenshot, **props)
end

#secure_storage(**props) ⇒ Object



1041
1042
1043
# File 'lib/ruflet_ui/ruflet/page.rb', line 1041

def secure_storage(**props)
  service(:secure_storage, **props)
end

#selection_click(timeout: 10, on_result: nil) ⇒ Object



898
899
900
# File 'lib/ruflet_ui/ruflet/page.rb', line 898

def selection_click(timeout: 10, on_result: nil)
  invoke_haptic_feedback("selection_click", timeout: timeout, on_result: on_result)
end

#semantics_service(**props) ⇒ Object



989
990
991
# File 'lib/ruflet_ui/ruflet/page.rb', line 989

def semantics_service(**props)
  service(:semantics_service, **props)
end

#service(type, **props) ⇒ Object



393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
# File 'lib/ruflet_ui/ruflet/page.rb', line 393

def service(type, **props)
  mapped_props = normalize_props(props || {})
  id = mapped_props.delete("id")
  normalized_type = type.to_s.downcase
  compact_type = normalized_type.delete("_")

  if visual_service_type?(normalized_type)
    key = id ? "id:#{id}" : normalized_type
    existing = @visual_service_controls[key]
    return existing if existing

    svc = Ruflet::UI::ControlFactory.build(type.to_s, id: id&.to_s, **mapped_props)
    @visual_service_controls[key] = svc
    return svc
  end

  existing =
    if id
      services.find { |s| s.is_a?(Control) && s.id.to_s == id.to_s }
    else
      services.find do |s|
        s.is_a?(Control) && s.type.to_s.downcase.delete("_") == compact_type
      end
    end
  return existing if existing

  svc = Ruflet::UI::ControlFactory.build(type.to_s, id: id&.to_s, **mapped_props)
  add_service(svc) unless services.include?(svc)
  svc
end

#servicesObject



310
311
312
# File 'lib/ruflet_ui/ruflet/page.rb', line 310

def services
  @services_container.props["_services"] ||= []
end

#services=(value) ⇒ Object



314
315
316
317
318
319
# File 'lib/ruflet_ui/ruflet/page.rb', line 314

def services=(value)
  @services_container.props["_services"] = Array(value).compact
  refresh_services_container!
  push_services_update!
  self
end

#set_clipboard(value, timeout: nil, on_result: nil) ⇒ Object



906
907
908
909
910
911
912
913
# File 'lib/ruflet_ui/ruflet/page.rb', line 906

def set_clipboard(value, timeout: nil, on_result: nil)
  invoke_clipboard_method(
    "set",
    args: { "data" => value.to_s },
    timeout: timeout,
    on_result: on_result
  )
end

#set_clipboard_files(files, timeout: nil, on_result: nil) ⇒ Object



919
920
921
922
923
924
925
926
# File 'lib/ruflet_ui/ruflet/page.rb', line 919

def set_clipboard_files(files, timeout: nil, on_result: nil)
  invoke_clipboard_method(
    "set_files",
    args: { "files" => Array(files).map(&:to_s) },
    timeout: timeout,
    on_result: on_result
  )
end

#set_clipboard_image(value, timeout: nil, on_result: nil) ⇒ Object



932
933
934
935
936
937
938
939
# File 'lib/ruflet_ui/ruflet/page.rb', line 932

def set_clipboard_image(value, timeout: nil, on_result: nil)
  invoke_clipboard_method(
    "set_image",
    args: { "data" => value },
    timeout: timeout,
    on_result: on_result
  )
end

#set_view_props(props) ⇒ Object



198
199
200
201
# File 'lib/ruflet_ui/ruflet/page.rb', line 198

def set_view_props(props)
  split_props(normalize_props(props || {}))
  self
end

#shake_detector(**props) ⇒ Object



985
986
987
# File 'lib/ruflet_ui/ruflet/page.rb', line 985

def shake_detector(**props)
  service(:shake_detector, **props)
end

#share(**props) ⇒ Object



1021
1022
1023
# File 'lib/ruflet_ui/ruflet/page.rb', line 1021

def share(**props)
  service(:share, **props)
end

#share_files(files = nil, text: nil, title: nil, subject: nil, preview_thumbnail: nil, share_position_origin: nil, download_fallback_enabled: true, mail_to_fallback_enabled: true, excluded_cupertino_activities: nil, timeout: nil, on_result: nil) ⇒ Object



1137
1138
1139
1140
1141
1142
1143
1144
1145
1146
1147
1148
1149
1150
1151
1152
1153
1154
1155
1156
1157
1158
1159
1160
1161
1162
1163
1164
1165
1166
1167
1168
# File 'lib/ruflet_ui/ruflet/page.rb', line 1137

def share_files(
  files = nil,
  text: nil,
  title: nil,
  subject: nil,
  preview_thumbnail: nil,
  share_position_origin: nil,
  download_fallback_enabled: true,
  mail_to_fallback_enabled: true,
  excluded_cupertino_activities: nil,
  timeout: nil,
  on_result: nil
)
  share = ensure_share_service
  invoke(
    share,
    "share_files",
    args: compact_service_args(
      "files" => normalize_share_files(files),
      "text" => text,
      "title" => title,
      "subject" => subject,
      "preview_thumbnail" => normalize_share_file(preview_thumbnail),
      "share_position_origin" => share_position_origin,
      "download_fallback_enabled" => download_fallback_enabled,
      "mail_to_fallback_enabled" => mail_to_fallback_enabled,
      "excluded_cupertino_activities" => excluded_cupertino_activities
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#share_text(text = nil, title: nil, subject: nil, preview_thumbnail: nil, share_position_origin: nil, download_fallback_enabled: true, mail_to_fallback_enabled: true, excluded_cupertino_activities: nil, timeout: nil, on_result: nil) ⇒ Object



1085
1086
1087
1088
1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
1111
1112
1113
1114
# File 'lib/ruflet_ui/ruflet/page.rb', line 1085

def share_text(
  text = nil,
  title: nil,
  subject: nil,
  preview_thumbnail: nil,
  share_position_origin: nil,
  download_fallback_enabled: true,
  mail_to_fallback_enabled: true,
  excluded_cupertino_activities: nil,
  timeout: nil,
  on_result: nil
)
  share = ensure_share_service
  invoke(
    share,
    "share_text",
    args: compact_service_args(
      "text" => text,
      "title" => title,
      "subject" => subject,
      "preview_thumbnail" => preview_thumbnail,
      "share_position_origin" => share_position_origin,
      "download_fallback_enabled" => download_fallback_enabled,
      "mail_to_fallback_enabled" => mail_to_fallback_enabled,
      "excluded_cupertino_activities" => excluded_cupertino_activities
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#share_uri(uri = nil, share_position_origin: nil, excluded_cupertino_activities: nil, timeout: nil, on_result: nil) ⇒ Object



1116
1117
1118
1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
# File 'lib/ruflet_ui/ruflet/page.rb', line 1116

def share_uri(
  uri = nil,
  share_position_origin: nil,
  excluded_cupertino_activities: nil,
  timeout: nil,
  on_result: nil
)
  share = ensure_share_service
  invoke(
    share,
    "share_uri",
    args: compact_service_args(
      "uri" => uri,
      "share_position_origin" => share_position_origin,
      "excluded_cupertino_activities" => excluded_cupertino_activities
    ),
    timeout: timeout,
    on_result: on_result
  )
end

#shared_preferences(**props) ⇒ Object



321
322
323
324
325
# File 'lib/ruflet_ui/ruflet/page.rb', line 321

def shared_preferences(**props)
  return service(:shared_preferences, **props) unless props.empty?

  @shared_preferences_proxy
end

#show_dialog(dialog_control) ⇒ Object



549
550
551
552
553
554
555
556
557
558
559
560
561
# File 'lib/ruflet_ui/ruflet/page.rb', line 549

def show_dialog(dialog_control)
  return self unless dialog_control

  return self if dialog_open?(dialog_control)

  dialog_control.props["open"] = true
  remove_existing_singleton_dialogs(dialog_control)
  @dialogs << dialog_control unless @dialogs.include?(dialog_control)
  refresh_dialogs_container!
  send_view_patch unless @dialogs_container.wire_id
  push_dialogs_update!
  self
end

#show_drawer(timeout: 10, on_result: nil) ⇒ Object

Raises:

  • (ArgumentError)


498
499
500
501
502
503
# File 'lib/ruflet_ui/ruflet/page.rb', line 498

def show_drawer(timeout: 10, on_result: nil)
  raise ArgumentError, "show_drawer requires drawer" unless drawer

  invoke_current_view("show_drawer", timeout: timeout, on_result: on_result)
  self
end

#show_end_drawer(timeout: 10, on_result: nil) ⇒ Object

Raises:

  • (ArgumentError)


510
511
512
513
514
515
# File 'lib/ruflet_ui/ruflet/page.rb', line 510

def show_end_drawer(timeout: 10, on_result: nil)
  raise ArgumentError, "show_end_drawer requires end_drawer" unless end_drawer

  invoke_current_view("show_end_drawer", timeout: timeout, on_result: on_result)
  self
end

#snack_bar=(value) ⇒ Object



530
531
532
533
534
# File 'lib/ruflet_ui/ruflet/page.rb', line 530

def snack_bar=(value)
  @snack_bar = value
  refresh_dialogs_container!
  push_dialogs_update! if @dialogs_container_mounted
end

#snackbar=(value) ⇒ Object



536
537
538
# File 'lib/ruflet_ui/ruflet/page.rb', line 536

def snackbar=(value)
  self.snack_bar = value
end

#start_window_dragging(timeout: 10, on_result: nil) ⇒ Object



753
754
755
# File 'lib/ruflet_ui/ruflet/page.rb', line 753

def start_window_dragging(timeout: 10, on_result: nil)
  invoke_window("start_dragging", timeout: timeout, on_result: on_result)
end

#start_window_resizing(edge, timeout: 10, on_result: nil) ⇒ Object



757
758
759
760
761
762
763
764
# File 'lib/ruflet_ui/ruflet/page.rb', line 757

def start_window_resizing(edge, timeout: 10, on_result: nil)
  invoke_window(
    "start_resizing",
    args: { "edge" => normalize_service_value(edge) },
    timeout: timeout,
    on_result: on_result
  )
end

#storage_paths(**props) ⇒ Object



1017
1018
1019
# File 'lib/ruflet_ui/ruflet/page.rb', line 1017

def storage_paths(**props)
  service(:storage_paths, **props)
end

#supports_close_for_launch_mode(mode, timeout: 10, on_result: nil) ⇒ Object



644
645
646
647
# File 'lib/ruflet_ui/ruflet/page.rb', line 644

def supports_close_for_launch_mode(mode, timeout: 10, on_result: nil)
  url_launcher = ensure_url_launcher_service
  invoke(url_launcher, "supports_close_for_launch_mode", args: { "mode" => mode }, timeout: timeout, on_result: on_result)
end

#supports_launch_mode(mode, timeout: 10, on_result: nil) ⇒ Object



639
640
641
642
# File 'lib/ruflet_ui/ruflet/page.rb', line 639

def supports_launch_mode(mode, timeout: 10, on_result: nil)
  url_launcher = ensure_url_launcher_service
  invoke(url_launcher, "supports_launch_mode", args: { "mode" => mode }, timeout: timeout, on_result: on_result)
end

#take_screenshot(name, timeout: 10, on_result: nil) ⇒ Object



796
797
798
# File 'lib/ruflet_ui/ruflet/page.rb', line 796

def take_screenshot(name, timeout: 10, on_result: nil)
  invoke_tester("take_screenshot", args: { "name" => name }, timeout: timeout, on_result: on_result)
end

#tap(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil) ⇒ Object



800
801
802
803
# File 'lib/ruflet_ui/ruflet/page.rb', line 800

def tap(finder_id = nil, options = nil, finder_index: nil, timeout: 10, on_result: nil)
  finder_index = options[:finder_index] || options["finder_index"] if options.is_a?(Hash) && finder_index.nil?
  invoke_tester_finder("tap", finder_id, finder_index: finder_index, timeout: timeout, on_result: on_result)
end

#tap_at(offset = nil, timeout: 10, on_result: nil) ⇒ Object



820
821
822
# File 'lib/ruflet_ui/ruflet/page.rb', line 820

def tap_at(offset = nil, timeout: 10, on_result: nil)
  invoke_tester_at("tap_at", offset, timeout: timeout, on_result: on_result)
end

#tester(**props) ⇒ Object



361
362
363
# File 'lib/ruflet_ui/ruflet/page.rb', line 361

def tester(**props)
  service(:tester, **props)
end

#tester_pump(options = nil, duration: nil, timeout: 10, on_result: nil) ⇒ Object



766
767
768
769
# File 'lib/ruflet_ui/ruflet/page.rb', line 766

def tester_pump(options = nil, duration: nil, timeout: 10, on_result: nil)
  duration = options[:duration] || options["duration"] if options.is_a?(Hash) && duration.nil?
  invoke_tester("pump", args: compact_service_args("duration" => duration), timeout: timeout, on_result: on_result)
end

#tester_pump_and_settle(options = nil, duration: nil, timeout: 10, on_result: nil) ⇒ Object



771
772
773
774
# File 'lib/ruflet_ui/ruflet/page.rb', line 771

def tester_pump_and_settle(options = nil, duration: nil, timeout: 10, on_result: nil)
  duration = options[:duration] || options["duration"] if options.is_a?(Hash) && duration.nil?
  invoke_tester("pump_and_settle", args: compact_service_args("duration" => duration), timeout: timeout, on_result: on_result)
end

#tester_teardown(timeout: 10, on_result: nil) ⇒ Object



882
883
884
# File 'lib/ruflet_ui/ruflet/page.rb', line 882

def tester_teardown(timeout: 10, on_result: nil)
  invoke_tester("teardown", timeout: timeout, on_result: on_result)
end

#titleObject



203
204
205
# File 'lib/ruflet_ui/ruflet/page.rb', line 203

def title
  @page_props["title"]
end

#title=(value) ⇒ Object



207
208
209
# File 'lib/ruflet_ui/ruflet/page.rb', line 207

def title=(value)
  @page_props["title"] = value
end

#update(control_or_id = nil, **props) ⇒ Object



1212
1213
1214
1215
1216
1217
1218
1219
1220
1221
1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
1233
1234
1235
1236
1237
1238
1239
1240
1241
1242
1243
1244
1245
1246
1247
1248
1249
1250
1251
1252
1253
1254
1255
1256
1257
# File 'lib/ruflet_ui/ruflet/page.rb', line 1212

def update(control_or_id = nil, **props)
  if control_or_id.nil? && props.empty?
    send_view_patch
    return self
  end

  if page_control_target?(control_or_id)
    split_props(normalize_props(props))
    send_view_patch
    return self
  end

  control = resolve_control(control_or_id)
  return self unless control
  wire_id = control.wire_id
  if wire_id.nil?
    # Events can race with navigation/disposal; never emit patch_control with nil id.
    refresh_control_indexes!
    wire_id = control.wire_id
  end
  return self if wire_id.nil?

  patch = normalize_props(props)
  if text_maps_to_content?(control, patch)
    patch["content"] = patch.delete("text")
  end

  # Keep runtime control tree aligned with incremental patches.
  if patch.key?("controls")
    control.children.clear
    Array(patch["controls"]).each { |child| control.children << child if child.is_a?(Control) }
  end

  visited = Set.new
  patch.each_value { |value| register_embedded_value(value, visited) }
  patch.each { |k, v| control.props[k] = v }

  patch_ops = patch.map { |k, v| [0, 0, k, serialize_patch_value(v)] }

  send_message(Protocol::ACTIONS[:patch_control], {
    "id" => wire_id,
    "patch" => [[0], *patch_ops]
  })

  self
end

#upload(files, timeout: nil, on_result: nil) ⇒ Object



712
713
714
715
716
717
718
719
# File 'lib/ruflet_ui/ruflet/page.rb', line 712

def upload(files, timeout: nil, on_result: nil)
  invoke_file_picker(
    "upload",
    { "files" => Array(files).map { |file| normalize_service_value(file) } },
    timeout: timeout,
    on_result: on_result
  )
end

#upload_files(files, timeout: nil, on_result: nil) ⇒ Object



721
722
723
# File 'lib/ruflet_ui/ruflet/page.rb', line 721

def upload_files(files, timeout: nil, on_result: nil)
  upload(files, timeout: timeout, on_result: on_result)
end

#url_launcher(**props) ⇒ Object



1013
1014
1015
# File 'lib/ruflet_ui/ruflet/page.rb', line 1013

def url_launcher(**props)
  service(:url_launcher, **props)
end

#user_accelerometer(**props) ⇒ Object



973
974
975
# File 'lib/ruflet_ui/ruflet/page.rb', line 973

def user_accelerometer(**props)
  service(:user_accelerometer, **props)
end

#vertical_alignmentObject



219
220
221
# File 'lib/ruflet_ui/ruflet/page.rb', line 219

def vertical_alignment
  @page_props["vertical_alignment"] || @view_props["vertical_alignment"]
end

#vertical_alignment=(value) ⇒ Object



223
224
225
226
227
# File 'lib/ruflet_ui/ruflet/page.rb', line 223

def vertical_alignment=(value)
  v = normalize_value("vertical_alignment", value)
  @page_props["vertical_alignment"] = v
  @view_props["vertical_alignment"] = v
end

#vibrate(timeout: 10, on_result: nil) ⇒ Object



902
903
904
# File 'lib/ruflet_ui/ruflet/page.rb', line 902

def vibrate(timeout: 10, on_result: nil)
  invoke_haptic_feedback("vibrate", timeout: timeout, on_result: on_result)
end

#wait_until_ready_to_show(timeout: 10, on_result: nil) ⇒ Object



733
734
735
# File 'lib/ruflet_ui/ruflet/page.rb', line 733

def wait_until_ready_to_show(timeout: 10, on_result: nil)
  invoke_window("wait_until_ready_to_show", timeout: timeout, on_result: on_result)
end

#wakelock(**props) ⇒ Object



327
328
329
330
331
# File 'lib/ruflet_ui/ruflet/page.rb', line 327

def wakelock(**props)
  return service(:wakelock, **props) unless props.empty?

  @wakelock_proxy
end

#wasmObject



275
276
277
# File 'lib/ruflet_ui/ruflet/page.rb', line 275

def wasm
  client_reported_prop("wasm")
end

#webObject



267
268
269
# File 'lib/ruflet_ui/ruflet/page.rb', line 267

def web
  client_reported_prop("web")
end

#widthObject

Client-reported page properties. The Flutter client sends these in its register payload (see Protocol.normalize_register_payload), where they are stored in @client_details; expose them as readers so apps can do ‘page.width`, `page.platform`, etc. without reaching into client_details.



251
252
253
# File 'lib/ruflet_ui/ruflet/page.rb', line 251

def width
  client_reported_prop("width")
end

#window(**props) ⇒ Object



357
358
359
# File 'lib/ruflet_ui/ruflet/page.rb', line 357

def window(**props)
  service(:window, **props)
end

#window_to_front(timeout: 10, on_result: nil) ⇒ Object



737
738
739
# File 'lib/ruflet_ui/ruflet/page.rb', line 737

def window_to_front(timeout: 10, on_result: nil)
  invoke_window("to_front", timeout: timeout, on_result: on_result)
end